
#include "TMath.h"

#include "GenericDAQBoard.h"


ClassImp(GenericDAQBoard)


//-----------------------------------------------------------------------
GenericDAQBoard::GenericDAQBoard() {
  nData = 0;
  iChan = 0;
  iValue = 0;
};

//-----------------------------------------------------------------------
GenericDAQBoard::GenericDAQBoard(UInt_t ndata, Int_t* channels, Int_t* values) {
  if (ndata<1) {
    nData = 0;
    iChan = 0;
    iValue = 0;
    return;
  };
  nData = ndata;
  iChan = new Int_t[ndata];
  iValue = new Int_t[ndata];
  for (UInt_t i=0; i<ndata; i++) {
    iChan[i] = channels[i];
    iValue[i] = values[i];
  };
};

//------------------------------------------------------------------------
GenericDAQBoard::~GenericDAQBoard() {
  ClearData();
};

//------------------------------------------------------------------------
Int_t GenericDAQBoard::FillData(UInt_t ndata, Int_t* channels, Int_t* values) {
  ClearData();
  if (ndata<1) return -1;
  nData = ndata;
  iChan = new Int_t[ndata];
  iValue = new Int_t[ndata];
  UInt_t i;
  for (i=0; i<ndata; i++) {
    iChan[i] = channels[i];
    iValue[i] = values[i];
  };
  return i;
};

void GenericDAQBoard::ClearData() {
  if (nData>0) {
    delete [] iChan;
    delete [] iValue;
    nData = 0;
  };
};

//------------------------------------------------------------------------
//Int_t GenericDAQBoard::GetAllData(Int_t* channels, Int_t* values) {};

//------------------------------------------------------------------------
Int_t GenericDAQBoard::GetChDatum(Int_t chan, UInt_t occurrence) {
  Int_t retval = -9999999;
  UInt_t count = 0;
  for (UInt_t i=0; i<nData; i++) {
    if (iChan[i] == chan) { 
      retval = iValue[i];
      count++;
    };
    if (count == occurrence) break;
  };
  return retval;
};

Int_t GenericDAQBoard::FindChDatum(Int_t chan, Int_t near, 
				   Int_t lowerbound, Int_t upperbound) {
  Int_t retval =   -9999999;
  UInt_t mindist = 9999999;
  for (UInt_t i=0; i<nData; i++) {

    if ((iChan[i] == chan) &&
	(iValue[i] > lowerbound) && (iValue[i] < upperbound) &&
	(TMath::Abs(iValue[i]-near) < mindist) ) {
      mindist = TMath::Abs(iValue[i]-near);
      retval = iValue[i];
    };

  };
  
  return retval;
  
};
