
{

  gROOT->Reset();
  
#define MY_DEBUG

  //////////////////////////////////////////////////////////////////////
  //
  // INPUT VARIABLES
  //  and input structures description
  //
  //////////////////////////////////////////////////////////////////////

  //input file
  char* rawfilename = "Run5part.txt";
  ifstream rawfile;

  // 128ch V1190A TDC descriptors 
#define V1190A_MAXIMUM_NDATA    4096
#define V1190A_HEADER_MARKER    0x8
#define V1190A_TRAILER_MARKER   0x10
#define V1190A_DATUM_MARKER     0x0
#define V1190A_ERROR_MARKER     0x4
  // 8ch V465 QDC descriptors 
#define V465_MAXIMUM_NDATA      8
#define V465_HEADER_MARKER     0x1
#define V465_DATUM_MARKER      0x0

  const UInt_t TDC1Address = 0x100000;
  const UInt_t TDC2Address = 0x110000;
  const UInt_t ADC1Address = 0x3000;





  //////////////////////////////////////////////////////////////////////
  //
  // OUTPUT VARIABLES
  //  and output structures description
  //
  //////////////////////////////////////////////////////////////////////

  if (!TClassTable::GetDict("GenericDAQBoard")) {
    gSystem->Load("GenericDAQBoard_cxx.so");
  }
  
  //Transient variables used to fill
  struct _general {
    Int_t nevt;
    Int_t trigtime;
    Int_t ntrig;
  } General;
 
  UInt_t TDC1ndata = 0;
  Int_t TDC1datach[V1190A_MAXIMUM_NDATA];
  Int_t TDC1dataval[V1190A_MAXIMUM_NDATA];
  UInt_t TDC2ndata = 0;
  Int_t TDC2datach[V1190A_MAXIMUM_NDATA];
  Int_t TDC2dataval[V1190A_MAXIMUM_NDATA];
  UInt_t ADC1ndata = 0;
  Int_t ADC1datach[V465_MAXIMUM_NDATA];
  Int_t ADC1dataval[V465_MAXIMUM_NDATA];

  class GenericDAQBoard* TDC1 = new GenericDAQBoard();
  class GenericDAQBoard* TDC2 = new GenericDAQBoard();
  class GenericDAQBoard* ADC1 = new GenericDAQBoard();

  //Output file
  TFile runfile("run5test.root", "recreate");

  //Output Ntuple declarations
  TTree CosmicRaysRun("Run","Cosmic Ray");    

  CosmicRaysRun.Branch("General", & General.nevt,"iEvt/I4:Trig/I4:nTrig/I2");
  CosmicRaysRun.Branch("TDC1n",   & TDC1ndata,   "TDC1n/i2");
  CosmicRaysRun.Branch("TDC1ch",  TDC1datach,    "TDC1ch[TDC1n]/I1");
  CosmicRaysRun.Branch("TDC1val", TDC1dataval,   "TDC1val[TDC1n]/I4");
  CosmicRaysRun.Branch("TDC2n",   & TDC2ndata,   "TDC2n/i2");
  CosmicRaysRun.Branch("TDC2ch",  TDC2datach,    "TDC2ch[TDC2n]/I1");
  CosmicRaysRun.Branch("TDC2val", TDC2dataval,   "TDC2val[TDC2n]/I4");
  CosmicRaysRun.Branch("ADC1n",   & ADC1ndata,   "ADC1n/i1");
  CosmicRaysRun.Branch("ADC1ch",  ADC1datach,    "ADC1ch[ADC1n]/I1");
  CosmicRaysRun.Branch("ADC1val", ADC1dataval,   "ADC1val[ADC1n]/I2");
  CosmicRaysRun.Branch("TDC1.", "GenericDAQBoard", & TDC1);
  //  CosmicRaysRun.Branch("TDC2.", "GenericDAQBoard", & TDC2);
  //CosmicRaysRun.Branch("ADC1.", "GenericDAQBoard", & ADC1);


  //////////////////////////////////////////////////////////////////////
  //
  // VARIABLES USED AS INDEXES and others
  //
  
  Int_t iEvt, iBoard, iDataType, iCh, iValue, iAdditional; 
  UInt_t iThisEvt = 0, iThisBoard = 0;
  UInt_t nRows;
  Int_t TDC1dataOK = 0, TDC2dataOK = 0, ADC1dataOK = 0;
  Bool_t firstevent = 1;
  
  //////////////////////////////////////////////////////////////////////
  //
  // START READING
  //
  
  rawfile.open(rawfilename);
  
  if (rawfile.is_open())
    {
      char filerow[200];
      while (rawfile.good())
	{
	  rawfile.getline(filerow,200);
	  if (sscanf(filerow,"%d %d %d %d %d %d",
		     &iEvt, &iBoard, &iDataType,
		     &iCh, &iValue, &iAdditional)
	      == 6) {
#ifdef MY_DEBUG
	    cout << iEvt << "\t" << iBoard << "\t" << iDataType << "\t" << iCh 
		 << "\t" << iValue << "\t" << iAdditional << endl << flush; //debug
	    
	    cin.get();
	    if (iEvt > 20) break;
#endif
	    
	    ///////////////////////////////////////////////////////////////////////////
	    //
	    //  NEW EVENT?
	    //
	    
	    if (iEvt > iThisEvt) { 
	      
	      //Write the previous event if it was OK
	      if ((!firstevent) && 
		  (TDC1dataOK == 2) &&
		  (TDC2dataOK == 2) &&
		  (ADC1dataOK == 2)) {
		TDC1.FillData(TDC1ndata, TDC1datach, TDC1dataval);
		TDC2.FillData(TDC2ndata, TDC2datach, TDC2dataval);
		ADC1.FillData(ADC1ndata, ADC1datach, ADC1dataval);
#ifdef MY_DEBUG
		cout << "TDC1 nData: " << "\t" << TDC1.nData << "\n";
		for (int i=0; i<TDC1.nData; i++)
		  cout << i << "\t" << TDC1->iChan[i] << "\t" << TDC1->iValue[i] << "\n"; //debug
		cout << "TDC2 nData: " << "\t" << TDC2.nData << "\n";
		for (int i=0; i<TDC2.nData; i++)
		  cout << i << "\t" << TDC2->iChan[i] << "\t" << TDC2->iValue[i] << "\n"; //debug
		cout << "ADC1 nData: " << "\t" << ADC1.nData << "\n";
		for (int i=0; i<ADC1.nData; i++)
		  cout << i << "\t" << ADC1->iChan[i] << "\t" << ADC1->iValue[i] << "\n"; //debug
		cout << endl;
		cin.get();
#endif
		CosmicRaysRun.Fill();
	      };
	      
	      firstevent = 0;
	      
	      //now next event, but which board?
	      if ((iBoard == TDC1Address) &&
		  (iDataType == V1190A_HEADER_MARKER)) {
		iThisEvt = iEvt;
		iThisBoard = iBoard;
		nRows = 1;
		General.nevt = iEvt;
		General.trigtime = 0;
		General.ntrig = 0;
		TDC1ndata = 0;
		TDC2ndata = 0;
		ADC1ndata = 0;
		TDC1dataOK = 1; //header found
		TDC2dataOK = 0;
		ADC1dataOK = 0;
	      }
	      else if ((iBoard == TDC2Address) &&
		       (iDataType == V1190A_HEADER_MARKER)) {
		iThisEvt = iEvt;
		iThisBoard = iBoard;
		nRows = 1;
		General.nevt = iEvt;
		General.trigtime = 0;
		General.ntrig = 0;
		TDC1ndata = 0;
		TDC2ndata = 0;
		ADC1ndata = 0;
		TDC1dataOK = 0;
		TDC2dataOK = 1; //header found
		ADC1dataOK = 0;
	      }
	      else if ((iBoard == ADC1Address) &&
		       (iDataType == V465_HEADER_MARKER)) {
		iThisEvt = iEvt;
		iThisBoard = iBoard;
		nRows = iCh;
		General.nevt = iEvt;
		General.trigtime = 0;
		General.ntrig = 0;
		TDC1ndata = 0;
		TDC2ndata = 0;
		ADC1ndata = 0;
		TDC1dataOK = 0;
		TDC2dataOK = 0;
		ADC1dataOK = 1; //header found
	      }
	      else 
		cout << "\n" << iThisEvt << ": new event data have started"
		     << " without any header" << endl;
	      
	    }
	    
	    ///////////////////////////////////////////////////////////////////  
	    //
	    // SAME EVENT?
	    //
	    
	    else if (iEvt == iThisEvt) {
	      
	      // TDC 1 ----------------------------------------------------------
	      if ((iBoard == TDC1Address) &&
		  (TDC1dataOK == -1)) {
		//an error already occurred in a previous dataword
		//of this event on this board. Skip all other datawords
		//of the board
	      }
	      else if ((iBoard == TDC1Address) &&
		       (TDC1dataOK == 0) &&         //neither header nor trailer found
		       (iDataType == V1190A_HEADER_MARKER)) {
		iThisBoard = iBoard;
		nRows = 1;
		TDC1ndata = 0;
		TDC1dataOK = 1; //header found
	      }
	      else if ((iBoard == TDC1Address) &&
		       (iThisBoard == iBoard) &&
		       (TDC1dataOK == 1) &&         //header found, trailer not
		       (iDataType == V1190A_DATUM_MARKER)) {
		if (TDC1ndata < V1190A_MAXIMUM_NDATA) {
		  TDC1datach[TDC1ndata] = iCh;
		  TDC1dataval[TDC1ndata] = iValue;
		  TDC1ndata++; nRows++;
		}
		else {
		  cout << "\n" << iThisEvt << ": Maximun number of data word "
		       << "exceeded in board " << iBoard 
		       << ". Some data are lost." << endl;
		  nRows++;
		};
	      }
	      else if ((iBoard == TDC1Address) &&
		       (iThisBoard == iBoard) &&
		       (TDC1dataOK == 1) &&         //header found, trailer not
		       (iDataType == V1190A_ERROR_MARKER)) {
		TDC1dataOK = -1; //an error has occurred. Mark this event as uncorrect
	      }
	      else if ((iBoard == TDC1Address) &&
		       (iThisBoard == iBoard) &&
		       (TDC1dataOK == 1) &&         //header found, trailer not
		       (iDataType == V1190A_TRAILER_MARKER)) {
		nRows++;
		if (nRows == iValue) {
		  TDC1dataOK = 2;
		}
		else {
		  TDC1dataOK = -1;
		  cout << "\n" << iThisEvt << ": a V1190A trailer is found "
		       << "without the respective header" << endl;		    
		};
	      }
	      
	      // TDC 2 ----------------------------------------------------------
	      else if ((iBoard == TDC2Address) &&
		  (TDC2dataOK == -1)) {
		//an error already occurred in a previous dataword
		//of this event on this board. Skip all other datawords
		//of the board
	      }
	      else if ((iBoard == TDC2Address) &&
		       (TDC2dataOK == 0) &&         //neither header nor trailer found
		       (iDataType == V1190A_HEADER_MARKER)) {
		iThisBoard = iBoard;
		nRows = 1;
		TDC2ndata = 0;
		TDC2dataOK = 1; //header found
	      }
	      else if ((iBoard == TDC2Address) &&
		       (iThisBoard == iBoard) &&
		       (TDC2dataOK == 1) &&         //header found, trailer not
		       (iDataType == V1190A_DATUM_MARKER)) {
		if (TDC2ndata < V1190A_MAXIMUM_NDATA) {
		  TDC2datach[TDC2ndata] = iCh;
		  TDC2dataval[TDC2ndata] = iValue;
		  TDC2ndata++; nRows++;
		}
		else {
		  cout << "\n" << iThisEvt << ": Maximun number of data word "
		       << "exceeded in board " << iBoard 
		       << ". Some data are lost." << endl;
		  nRows++;
		};
	      }
	      else if ((iBoard == TDC2Address) &&
		       (iThisBoard == iBoard) &&
		       (TDC2dataOK == 1) &&         //header found, trailer not
		       (iDataType == V1190A_ERROR_MARKER)) {
		TDC2dataOK = -1; //an error has occurred. Mark this event as uncorrect
	      }
	      else if ((iBoard == TDC2Address) &&
		       (iThisBoard == iBoard) &&
		       (TDC2dataOK == 1) &&         //header found, trailer not
		       (iDataType == V1190A_TRAILER_MARKER)) {
		nRows++;
		if (nRows == iValue) {
		  TDC2dataOK = 2;
		}
		else {
		  TDC2dataOK = -1;
		  cout << "\n" << iThisEvt << ": a V1190A trailer is found "
		       << "without the respective header" << endl;
		};
	      }
	      
	      // ADC 1 ---------------------------------------------------------
	      else if ((iBoard == ADC1Address) &&
		  (ADC1dataOK == -1)) {
		//an error already occurred in a previous dataword
		//of this event on this board. Skip all other datawords
		//of the board
	      }
	      else if ((iBoard == ADC1Address) &&
		       (iDataType == V465_HEADER_MARKER)) {
		iThisBoard = iBoard;
		nRows = iCh;
		ADC1ndata = 0;
		ADC1dataOK = 1; //header found
	      }
	      else if ((iBoard == ADC1Address) &&
		       (iThisBoard == iBoard) &&
		       (ADC1dataOK >= 1) &&         //header found
		       (iDataType == V465_DATUM_MARKER)) {
		if (ADC1ndata < V465_MAXIMUM_NDATA) {
		  ADC1datach[ADC1ndata] = iCh;
		  ADC1dataval[ADC1ndata] = iValue;
		  ADC1ndata++;
		  if (ADC1ndata == nRows) //number of data read is the expected one
		    ADC1dataOK = 2; 
		  if (ADC1ndata > nRows) { 
		    ADC1dataOK = -1; 
		    cout << "\n" << iThisEvt << ": more V465 datawords than the "
			 << "number declared in its header" << endl;
		  };
		}
		else { //this should never occur (ADC isn't "multihit")
		  cout << "\n" << iThisEvt << ": Maximun number of data word "
		       << "exceeded in board " << iBoard 
		       << ". Some data are lost." << endl;
		};
	      }
	      
	      // ALL OTHER CASES ARE ERRORS!! -------------------------------------
	      else {
		cout << "\n" << iThisEvt << ": Unrecognized dataword:"
		     << "\t\t" << filerow << endl;
		TDC1dataOK = -1;
		TDC2dataOK = -1;
		ADC1dataOK = -1;
	      };
	    }
	    
	    ///////////////////////////////////////////////////////////////////////
	    //
	    // EVENT NUMBER LESS THAN THE PREVIOUS OR THAN ZERO?  (?!?!??!)		
	    //
	    
	    else {
	      //Simply skip these cases
	      cout << "\nFound event " << iEvt << "after event" << iThisEvt 
		   << "Skipping all data of this event" << endl;
	    };
	  }
	  else {
	    cout << "\n" << iEvt << ": Unrecognized dataword during sscanf:"
		 << "\n" << filerow << "\n" << endl;

	  }; //end of filerow scanning

	}; //end of while(rawfile.good())
      
      if(rawfile.eof())
	cout << "The whole file has been read without I/O error" << endl;

    } //end of true case of if(rawfile.is_open())
  else
    {
      cout << "Error opening file" << endl;
    }; //end of
  
  rawfile.close();

  CosmicRaysRun.Write();
  CosmicRaysRun.Print();
}
